from typing import TYPE_CHECKING, Any

from django.contrib.admin import helpers
from django.contrib.admin.utils import lookup_field, quote
from django.core.exceptions import ObjectDoesNotExist
from django.db import models
from django.db.models import (
    Field,
    FileField,
    ForeignObjectRel,
    ImageField,
    JSONField,
    ManyToManyRel,
    OneToOneField,
)
from django.forms import ModelChoiceField, ModelMultipleChoiceField, Widget
from django.forms.utils import flatatt
from django.template.defaultfilters import linebreaksbr
from django.urls import NoReverseMatch, reverse, reverse_lazy
from django.utils.html import conditional_escape, format_html
from django.utils.module_loading import import_string
from django.utils.safestring import SafeString, SafeText, mark_safe
from django.utils.text import capfirst

from unfold.settings import get_config
from unfold.utils import display_for_field, prettify_json
from unfold.widgets import (
    CHECKBOX_LABEL_CLASSES,
    LABEL_CLASSES,
    UnfoldAdminAutocompleteModelChoiceFieldWidget,
    UnfoldAdminMultipleAutocompleteModelChoiceFieldWidget,
)

if TYPE_CHECKING:
    from unfold.admin import ModelAdmin


class UnfoldAdminReadonlyField(helpers.AdminReadonlyField):
    model_admin: "ModelAdmin"

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)

        self.resolved_field = self._resolve_field()

    def label_tag(self) -> SafeText:
        attrs = {
            "class": " ".join(LABEL_CLASSES + ["mb-2"]),
        }

        label = self.field["label"]

        return format_html("<label{}>{}</label>", flatatt(attrs), capfirst(label))

    @property
    def url(self) -> str | bool:
        if not self.is_file:
            return False

        field_name = self.field["field"]
        if isinstance(field_name, str) and hasattr(self.form.instance, field_name):
            field_value = getattr(self.form.instance, field_name)

            if field_value and hasattr(field_value, "url"):
                return field_value.url

        return False

    @property
    def is_json(self) -> bool:
        if isinstance(self.resolved_field, bool) or not self.resolved_field:
            return False

        f, attr, value = self.resolved_field

        return isinstance(f, JSONField)

    @property
    def is_image(self) -> bool:
        if isinstance(self.resolved_field, bool) or not self.resolved_field:
            return False

        f, attr, value = self.resolved_field

        return isinstance(f, ImageField)

    @property
    def is_file(self) -> bool:
        if isinstance(self.resolved_field, bool) or not self.resolved_field:
            return False

        f, attr, value = self.resolved_field
        return isinstance(f, ImageField | FileField)

    def contents(self) -> SafeString:
        contents = self._get_contents()
        contents = self._preprocess_field(contents)
        return contents

    def get_admin_url(self, remote_field, remote_obj):
        url_name = f"admin:{remote_field.model._meta.app_label}_{remote_field.model._meta.model_name}_change"
        try:
            url = reverse(
                url_name,
                args=[quote(remote_obj.pk)],
                current_app=self.model_admin.admin_site.name,
            )
            return format_html('<a href="{}" class="text-link">{}</a>', url, remote_obj)
        except NoReverseMatch:
            return str(remote_obj)

    def _get_contents(self) -> SafeString:  # noqa: PLR0912
        from unfold.utils import _boolean_icon

        field, obj, model_admin = (
            self.field["field"],
            self.form.instance,
            self.model_admin,
        )
        try:
            f, attr, value = lookup_field(field, obj, model_admin)
        except (AttributeError, ValueError, ObjectDoesNotExist):
            result_repr = self.empty_value_display
        else:
            if isinstance(field, str) and field in self.form.fields:
                widget = self.form[field].field.widget
                # This isn't elegant but suffices for contrib.auth's
                # ReadOnlyPasswordHashWidget.
                if getattr(widget, "read_only", False):
                    return widget.render(field, value)

            if f is None:
                if getattr(attr, "boolean", False):
                    result_repr = _boolean_icon(value)
                elif hasattr(value, "__html__"):
                    result_repr = value
                else:
                    result_repr = linebreaksbr(value)
            else:
                if isinstance(f.remote_field, ManyToManyRel) and value is not None:
                    result_repr = ", ".join(map(str, value.all()))
                elif (
                    isinstance(f.remote_field, ForeignObjectRel | OneToOneField)
                    and value is not None
                ):
                    result_repr = self.get_admin_url(f.remote_field, value)
                elif isinstance(f, models.JSONField):
                    formatted_output = prettify_json(value, f.encoder)

                    if formatted_output:
                        return formatted_output

                    result_repr = display_for_field(value, f, self.empty_value_display)
                    return conditional_escape(result_repr)
                elif isinstance(f, models.URLField):
                    return value and format_html(
                        '<a href="{}" class="text-link">{}</a>', value, value
                    )
                else:
                    result_repr = display_for_field(value, f, self.empty_value_display)
                    return conditional_escape(result_repr)
                result_repr = linebreaksbr(result_repr)
        return conditional_escape(result_repr)

    def _preprocess_field(self, contents: SafeString) -> SafeString:
        if (
            hasattr(self.model_admin, "readonly_preprocess_fields")
            and self.field["field"]
            in self.model_admin.readonly_preprocess_fields.keys()
        ):
            func = self.model_admin.readonly_preprocess_fields[self.field["field"]]
            if isinstance(func, str):
                contents = import_string(func)(contents)
            elif callable(func):
                contents = func(contents)

        return contents

    def _resolve_field(self) -> bool | tuple[Field | None, str | None, Any]:
        field, obj, model_admin = (
            self.field["field"],
            self.form.instance,
            self.model_admin,
        )

        try:
            return lookup_field(field, obj, model_admin)
        except (AttributeError, ValueError, ObjectDoesNotExist):
            pass

        return False


class UnfoldAdminField(helpers.AdminField):
    def label_tag(self) -> SafeText:
        classes = []

        # TODO load config from current AdminSite (override Fieldline.__iter__ method)
        flags = get_config()["EXTENSIONS"]["modeltranslation"]["flags"]

        for lang, flag in flags.items():
            if f"[{lang}]" in self.field.label:
                self.field.label = self.field.label.replace(f"[{lang}]", flag)
                break

        contents = conditional_escape(self.field.label)

        classes.append(
            " ".join(CHECKBOX_LABEL_CLASSES if self.is_checkbox else LABEL_CLASSES)
        )

        if self.field.field.required:
            classes.append("required")

        attrs = {"class": " ".join(classes)} if classes else {}
        required = mark_safe(' <span class="text-red-600">*</span>')

        return self.field.label_tag(
            contents=mark_safe(contents),
            attrs=attrs,
            label_suffix=required if self.field.field.required else "",
        )


class AutocompleteFieldMixin:
    def __init__(self, url_path: str, *args: Any, **kwargs: Any) -> None:
        self.url_path = url_path
        super().__init__(*args, **kwargs)

    def widget_attrs(self, widget: Widget) -> dict[str, Any]:
        return {
            "data-ajax--url": reverse_lazy(self.url_path),
        }


class UnfoldAdminAutocompleteModelChoiceField(AutocompleteFieldMixin, ModelChoiceField):
    widget = UnfoldAdminAutocompleteModelChoiceFieldWidget


class UnfoldAdminMultipleAutocompleteModelChoiceField(
    AutocompleteFieldMixin, ModelMultipleChoiceField
):
    widget = UnfoldAdminMultipleAutocompleteModelChoiceFieldWidget



from collections.abc import Callable, Iterator
from typing import Any

from django.contrib.admin import (
    ChoicesFieldListFilter,
    ListFilter,
    RelatedFieldListFilter,
)
from django.contrib.admin.options import ModelAdmin
from django.contrib.admin.views.main import ChangeList
from django.db.models import QuerySet
from django.db.models.fields import BLANK_CHOICE_DASH
from django.db.models.fields.related import RelatedField
from django.forms import ValidationError
from django.http import HttpRequest
from django.utils.translation import gettext_lazy as _

from unfold.contrib.filters.forms import (
    AutocompleteDropdownForm,
    CheckboxForm,
    DropdownForm,
    RadioForm,
    RangeNumericForm,
)


class ValueMixin:
    lookup_val = None

    def value(self) -> str | None:
        if isinstance(self.lookup_val, list) and len(self.lookup_val):
            return self.lookup_val[0]

        return self.lookup_val


class MultiValueMixin:
    lookup_val = None

    def value(self) -> list[str] | None:
        return self.lookup_val


class DropdownMixin:
    template = "unfold/filters/filters_field.html"
    form_class = DropdownForm
    all_option = ["", _("All")]


class ChoicesMixin(ChoicesFieldListFilter):
    template = "unfold/filters/filters_field.html"
    all_option: tuple[str, str] | None = None
    form_class: type[CheckboxForm | RadioForm]
    value: Callable

    def choices(self, changelist: ChangeList) -> Iterator:
        add_facets = getattr(changelist, "add_facets", False)
        facet_counts = self.get_facet_queryset(changelist) if add_facets else None
        choices = [self.all_option] if self.all_option else []

        for i, choice in enumerate(self.field.flatchoices):
            if add_facets and facet_counts:
                count = facet_counts[f"{i}__c"]
                choices.append((choice[0], f"{choice[1]} ({count})"))
            else:
                choices.append(choice)

        yield {
            "form": self.form_class(
                label=_(" By %(filter_title)s ") % {"filter_title": self.title},
                name=self.lookup_kwarg,
                choices=choices,
                data={
                    self.lookup_kwarg: self.value(),
                },
            ),
        }


class RangeNumericMixin(ListFilter):
    request = None
    template = "unfold/filters/filters_numeric_range.html"
    parameter_name: str | None = None

    def init_used_parameters(self, params: dict[str, Any]) -> None:
        if f"{self.parameter_name}_from" in params:
            value = params.pop(f"{self.parameter_name}_from")

            self.used_parameters[f"{self.parameter_name}_from"] = (
                value[0] if isinstance(value, list) else value
            )

        if f"{self.parameter_name}_to" in params:
            value = params.pop(f"{self.parameter_name}_to")
            self.used_parameters[f"{self.parameter_name}_to"] = (
                value[0] if isinstance(value, list) else value
            )

    def queryset(self, request: HttpRequest, queryset: QuerySet) -> QuerySet | None:
        filters = {}

        value_from = self.used_parameters.get(f"{self.parameter_name}_from", None)
        if value_from is not None and value_from != "":
            filters.update(
                {
                    f"{self.parameter_name}__gte": self.used_parameters.get(
                        f"{self.parameter_name}_from", None
                    ),
                }
            )

        value_to = self.used_parameters.get(f"{self.parameter_name}_to", None)
        if value_to is not None and value_to != "":
            filters.update(
                {
                    f"{self.parameter_name}__lte": self.used_parameters.get(
                        f"{self.parameter_name}_to", None
                    ),
                }
            )

        try:
            return queryset.filter(**filters)
        except (ValueError, ValidationError):
            return None

    def expected_parameters(self) -> list[str | None]:
        return [
            f"{self.parameter_name}_from",
            f"{self.parameter_name}_to",
        ]

    def choices(self, changelist: ChangeList) -> Iterator:
        if self.parameter_name:
            yield {
                "request": self.request,
                "parameter_name": self.parameter_name,
                "form": RangeNumericForm(
                    name=self.parameter_name,
                    data={
                        f"{self.parameter_name}_from": self.used_parameters.get(
                            f"{self.parameter_name}_from", None
                        ),
                        f"{self.parameter_name}_to": self.used_parameters.get(
                            f"{self.parameter_name}_to", None
                        ),
                    },
                ),
            }


class AutocompleteMixin(RelatedFieldListFilter):
    model_admin: ModelAdmin
    form_class: type[AutocompleteDropdownForm]
    value: Callable

    def has_output(self) -> bool:
        return True

    def field_choices(
        self, field: RelatedField, request: HttpRequest, model_admin: ModelAdmin
    ) -> list[tuple]:
        return [
            ("", BLANK_CHOICE_DASH),
        ]

    def choices(self, changelist: ChangeList) -> Iterator:
        yield {
            "form": self.form_class(
                request=self.request,
                label=_(" By %(filter_title)s ") % {"filter_title": self.title},
                name=self.lookup_kwarg,
                choices=(),
                field=self.field,
                model_admin=self.model_admin,
                data={
                    self.lookup_kwarg: self.value(),
                },
                multiple=self.multiple
                if hasattr(self, "multiple") and isinstance(self.multiple, bool)
                else False,
            ),
        }



from typing import Any

import django
from django.contrib import messages
from django.contrib.admin.filters import ListFilter
from django.contrib.admin.views.main import ERROR_FLAG, PAGE_VAR
from django.contrib.admin.views.main import ChangeList as BaseChangeList
from django.contrib.auth.mixins import PermissionRequiredMixin
from django.db.models import QuerySet
from django.http import HttpRequest, JsonResponse
from django.views.generic import ListView

from unfold.exceptions import UnfoldException
from unfold.forms import DatasetChangeListSearchForm


class ChangeList(BaseChangeList):
    def __init__(self, request: HttpRequest, *args: Any, **kwargs: Any) -> None:
        super().__init__(request, *args, **kwargs)

        if django.VERSION < (5, 0):
            self.filter_params = dict(request.GET.lists())
            self.filter_params.pop(PAGE_VAR, None)
            self.filter_params.pop(ERROR_FLAG, None)


class DatasetChangeList(ChangeList):
    is_dataset = True

    def __init__(self, request: HttpRequest, *args: Any, **kwargs: Any) -> None:
        self.search_var = f"{kwargs.get('model')._meta.model_name}-q"
        self.page_var = f"{kwargs.get('model')._meta.model_name}-p"

        _search_form = DatasetChangeListSearchForm(
            request.GET, search_var=self.search_var
        )
        if not _search_form.is_valid():
            for error in _search_form.errors.values():
                messages.error(request, ", ".join(error))

        self.dataset_search_query = _search_form.cleaned_data.get(self.search_var) or ""

        super().__init__(request, *args, **kwargs)

    def get_results(self, request: HttpRequest) -> None:
        try:
            self.page_num = int(request.GET.get(self.page_var, 1))
        except ValueError:
            self.page_num = 1

        super().get_results(request)

    def get_queryset(
        self, request: HttpRequest, exclude_parameters: list[str | None] | None = None
    ) -> QuerySet:
        self.query = self.dataset_search_query
        return super().get_queryset(request, exclude_parameters)

    def get_filters(
        self, request: str
    ) -> tuple[list[ListFilter], bool, dict[str, bool | str], bool, bool]:
        # Disable filters for dataset
        return ([], False, {}, False, False)


class UnfoldModelAdminViewMixin(PermissionRequiredMixin):
    """
    Prepares views to be displayed in admin
    """

    model_admin = None

    def __init__(self, model_admin, **kwargs):
        self.model_admin = model_admin
        super().__init__(**kwargs)

    def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
        self.request.current_app = self.model_admin.admin_site.name

        if not hasattr(self, "model_admin"):
            raise UnfoldException(
                "UnfoldModelAdminViewMixin was not provided with 'model_admin' argument"
            )

        if not hasattr(self, "title"):
            raise UnfoldException(
                "UnfoldModelAdminViewMixin was not provided with 'title' attribute"
            )

        return super().get_context_data(
            **kwargs,
            **self.model_admin.admin_site.each_context(self.request),
            **{
                "title": self.title,
                "model_admin": self.model_admin,
            },
        )


class BaseAutocompleteView(ListView):
    paginate_by = 20

    def get(self, request: HttpRequest, *args: Any, **kwargs: Any) -> JsonResponse:
        super().get(request, *args, **kwargs)
        context = self.get_context_data()

        return JsonResponse(
            {
                "results": [
                    {
                        "id": str(obj.pk),
                        "text": str(obj),
                    }
                    for obj in context["object_list"]
                ],
                "pagination": {
                    "more": context["page_obj"].has_next(),
                },
            }
        )



from functools import update_wrapper
from typing import Any

from django import forms
from django.contrib.admin import ModelAdmin as BaseModelAdmin
from django.contrib.admin import StackedInline as BaseStackedInline
from django.contrib.admin import TabularInline as BaseTabularInline
from django.contrib.admin import display, helpers
from django.contrib.admin.options import InlineModelAdmin
from django.contrib.contenttypes.admin import (
    GenericStackedInline as BaseGenericStackedInline,
)
from django.contrib.contenttypes.admin import (
    GenericTabularInline as BaseGenericTabularInline,
)
from django.db.models import BLANK_CHOICE_DASH, Model
from django.http import HttpRequest, HttpResponse
from django.shortcuts import redirect
from django.template.response import TemplateResponse
from django.urls import URLPattern, path
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from django.views import View

from unfold.checks import UnfoldModelAdminChecks
from unfold.datasets import BaseDataset
from unfold.forms import (
    ActionForm,
    PaginationGenericInlineFormSet,
    PaginationInlineFormSet,
)
from unfold.mixins import (
    ActionModelAdminMixin,
    BaseModelAdminMixin,
    DatasetModelAdminMixin,
)
from unfold.overrides import FORMFIELD_OVERRIDES_INLINE
from unfold.typing import FieldsetsType
from unfold.views import ChangeList
from unfold.widgets import UnfoldBooleanWidget

checkbox = UnfoldBooleanWidget(
    {
        "class": "action-select",
        "aria-label": _("Select record"),
    },
    lambda value: False,
)


class ModelAdmin(
    BaseModelAdminMixin, ActionModelAdminMixin, DatasetModelAdminMixin, BaseModelAdmin
):
    action_form = ActionForm
    custom_urls = ()
    add_fieldsets = ()
    ordering_field = None
    hide_ordering_field = False
    list_horizontal_scrollbar_top = False
    list_filter_submit = False
    list_filter_sheet = True
    list_fullwidth = False
    list_disable_select_all = False
    list_before_template = None
    list_after_template = None
    change_form_before_template = None
    change_form_after_template = None
    change_form_outer_before_template = None
    change_form_outer_after_template = None
    change_form_datasets = ()
    compressed_fields = False
    show_add_link = True
    readonly_preprocess_fields = {}
    warn_unsaved_form = False
    checks_class = UnfoldModelAdminChecks

    @property
    def media(self):
        if not hasattr(self, "request"):
            return super().media

        media = super().media

        for filter in self.get_list_filter(self.request):
            if (
                isinstance(filter, tuple | list)
                and hasattr(filter[1], "form_class")
                and hasattr(filter[1].form_class, "Media")
            ):
                media += forms.Media(filter[1].form_class.Media)
            elif hasattr(filter, "form_class") and hasattr(filter.form_class, "Media"):
                media += forms.Media(filter.form_class.Media)

        return media

    def changelist_view(
        self, request: HttpRequest, extra_context: dict[str, str] | None = None
    ) -> TemplateResponse:
        self.request = request

        if self.ordering_field and self.ordering_field not in self.list_editable:
            list_editable = list(getattr(self, "list_editable", []))
            list_editable.append(self.ordering_field)
            self.list_editable = list_editable

        return super().changelist_view(request, extra_context)

    def get_list_display(self, request: HttpRequest) -> list | tuple:
        list_display = super().get_list_display(request)

        if self.ordering_field and self.ordering_field not in list_display:
            if isinstance(list_display, tuple):
                list_display = (*list_display, self.ordering_field)
            elif isinstance(list_display, list):
                list_display.append(self.ordering_field)

        return list_display

    def get_fieldsets(
        self, request: HttpRequest, obj: Model | None = None
    ) -> FieldsetsType:
        if not obj and self.add_fieldsets:
            return self.add_fieldsets
        return super().get_fieldsets(request, obj)

    def get_custom_urls(self) -> tuple[tuple[str, str, View], ...]:
        """
        Method to get custom views for ModelAdmin with their urls

        Format of custom_urls item:
            ("path_to_view", "name_of_view", view_itself)
        """
        return () if self.custom_urls is None else self.custom_urls

    def get_urls(self) -> list[URLPattern]:
        urls = super().get_urls()

        def wrap(view):
            def wrapper(*args, **kwargs):
                return self.admin_site.admin_view(view)(*args, **kwargs)

            wrapper.model_admin = self
            return update_wrapper(wrapper, view)

        custom_urls = [
            self._path_from_custom_url(custom_url)
            for custom_url in self.get_custom_urls()
        ]

        actions_list_urls = [
            path(
                f"{action.path.removesuffix('/')}/",
                wrap(action.method),
                name=action.action_name,
            )
            for action in self._get_base_actions_list()
        ]

        action_detail_urls = [
            path(
                f"<path:object_id>/{action.path.removesuffix('/')}/",
                wrap(action.method),
                name=action.action_name,
            )
            for action in self._get_base_actions_detail()
        ]

        action_row_urls = [
            path(
                f"<path:object_id>/{action.path.removesuffix('/')}/",
                wrap(action.method),
                name=action.action_name,
            )
            for action in self._get_base_actions_row()
        ]

        return (
            custom_urls
            + action_row_urls
            + actions_list_urls
            + action_detail_urls
            + urls
        )

    def _path_from_custom_url(self, custom_url: tuple[str, str, View]) -> URLPattern:
        return path(
            custom_url[0],
            self.admin_site.admin_view(custom_url[2]),
            {"model_admin": self},
            name=custom_url[1],
        )

    def get_action_choices(
        self,
        request: HttpRequest,
        default_choices: list[tuple[str, str]] = BLANK_CHOICE_DASH,
    ) -> list[tuple[str, str]]:
        default_choices = [("", _("Select action"))]
        return super().get_action_choices(request, default_choices)

    @display(description=mark_safe(checkbox.render("action_toggle_all", 1)))
    def action_checkbox(self, obj: Model) -> str:
        return checkbox.render(helpers.ACTION_CHECKBOX_NAME, str(obj.pk))

    def response_change(self, request: HttpRequest, obj: Model) -> HttpResponse:
        res = super().response_change(request, obj)
        if "next" in request.GET:
            return redirect(request.GET["next"])
        return res

    def response_add(
        self, request: HttpRequest, obj: Model, post_url_continue: str | None = None
    ) -> HttpResponse:
        res = super().response_add(request, obj, post_url_continue)
        if "next" in request.GET:
            return redirect(request.GET["next"])
        return res

    def get_changelist(self, request: HttpRequest, **kwargs: Any) -> ChangeList:
        return ChangeList

    def get_formset_kwargs(
        self, request: HttpRequest, obj: Model, inline: InlineModelAdmin, prefix: str
    ) -> dict[str, Any]:
        formset_kwargs = super().get_formset_kwargs(request, obj, inline, prefix)

        if hasattr(inline, "per_page") and inline.per_page:
            formset_kwargs["request"] = request
            formset_kwargs["per_page"] = inline.per_page

        if hasattr(inline, "show_count") and inline.show_count:
            if hasattr(inline, "get_count") and callable(inline.get_count):
                formset_kwargs["count"] = inline.get_count(request, obj)

            if hasattr(inline, "get_count_variant") and callable(
                inline.get_count_variant
            ):
                formset_kwargs["count_variant"] = inline.get_count_variant(request, obj)

        return formset_kwargs

    def get_changeform_datasets(self, request: HttpRequest) -> list[type[BaseDataset]]:
        return self.change_form_datasets


class BaseInlineMixin:
    formfield_overrides = FORMFIELD_OVERRIDES_INLINE
    readonly_preprocess_fields = {}
    ordering_field = None
    per_page = None
    hide_ordering_field = False
    collapsible = False
    show_count = False
    hide_title = False
    tab = False


class TabularInline(BaseInlineMixin, BaseModelAdminMixin, BaseTabularInline):
    formset = PaginationInlineFormSet


class StackedInline(BaseInlineMixin, BaseModelAdminMixin, BaseStackedInline):
    formset = PaginationInlineFormSet


class GenericStackedInline(
    BaseInlineMixin, BaseModelAdminMixin, BaseGenericStackedInline
):
    formset = PaginationGenericInlineFormSet


class GenericTabularInline(
    BaseInlineMixin, BaseModelAdminMixin, BaseGenericTabularInline
):
    formset = PaginationGenericInlineFormSet



from collections.abc import Callable
from dataclasses import dataclass

from unfold.enums import ActionVariant
from unfold.typing import ActionFunction


@dataclass(frozen=True)
class UnfoldAction:
    action_name: str
    method: ActionFunction
    description: str
    path: str
    attrs: dict | None = None
    object_id: int | str | None = None
    icon: str | None = None
    variant: ActionVariant | None = ActionVariant.DEFAULT


@dataclass
class SearchResult:
    title: str
    description: str
    link: str
    icon: str | None


@dataclass
class Favicon:
    href: str | Callable
    rel: str | None = None
    type: str | None = None
    sizes: str | None = None


@dataclass
class DropdownItem:
    title: str
    link: str | Callable
    icon: str | None = None
    attrs: dict | None = None
